# Copyright 2020 The Merlin Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import shutil
import pathlib

try:
    import importlib.resources as pkg_resources
except ImportError:
    # Try backported to PY<37 `importlib_resources`.

    import importlib_resources as pkg_resources  # type: ignore


def copy_pyfunc_dockerfile(dst):
    """
    Copy pyfunc dockerfile to the destination path

    :param dst: destination path
    :return: full path to pyfunc Dockerfile
    """
    return _copy_dockerfile(dst, "pyfunc.Dockerfile")


def copy_standard_dockerfile(dst):
    """
    Copy dockerfile for standarnd model to the destination path

    :param dst: detination path
    :return: full path to standard model Dockerfile
    """
    return _copy_dockerfile(dst, "standard.Dockerfile")


def _copy_dockerfile(dst, dockerfile_name):
    dst_dockerfile_path = os.path.join(dst, dockerfile_name)
    if os.path.isfile(dst_dockerfile_path):
        return dst_dockerfile_path

    absolute_package_path = pathlib.Path(__file__).absolute()
    package_path = os.path.dirname(absolute_package_path)
    full_path = os.path.join(package_path, dockerfile_name)
    shutil.copy(full_path, dst_dockerfile_path)
    return dst_dockerfile_path



import enum


class Protocol(enum.Enum):
    """
    Model deployment protocol.

    HTTP_JSON = Deploy model and expose HTTP server
    UPI_V1 = Deploy model and expose gRPC server compatible with universal-prediction-interface (https://github.com/caraml-dev/universal-prediction-interface)
    """
    HTTP_JSON = "HTTP_JSON"
    UPI_V1 = "UPI_V1"



# Copyright 2020 The Merlin Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from merlin.validation import validate_model_dir
from merlin.model import ModelType
import pytest
import re

@pytest.mark.parametrize(
    "model_type,model_dir,message",
    [
        (ModelType.PYTORCH, "test/invalid-models/pytorch-model-invalid", "test/invalid-models/pytorch-model-invalid/config/config.properties is not found"),
        (ModelType.TENSORFLOW, "test/invalid-models/tensorflow-model-invalid", "test/invalid-models/tensorflow-model-invalid/1/saved_model.pb is not found"),
        (ModelType.SKLEARN, "test/invalid-models/sklearn-model-invalid", "['model.joblib'] is not found in test/invalid-models/sklearn-model-invalid"),
        (ModelType.XGBOOST, "test/invalid-models/xgboost-model-invalid", "['model.bst'] is not found in test/invalid-models/xgboost-model-invalid"),
        (ModelType.ONNX, "test/invalid-models/onnx-model-invalid", "['model.onnx'] is not found in test/invalid-models/onnx-model-invalid")
    ]
)
@pytest.mark.unit
def test_invalid_model_dir(model_type, model_dir, message):
    with pytest.raises(ValueError, match=re.escape(message)):
        validate_model_dir(model_type, model_dir)


@pytest.mark.parametrize(
    "model_type,model_dir",
    [
        (ModelType.PYTORCH, "test/pytorch-model/pytorch-sample"),
        (ModelType.TENSORFLOW, "test/tensorflow-model"),
        (ModelType.SKLEARN, "test/sklearn-model"),
        (ModelType.XGBOOST, "test/xgboost-model"),
        (ModelType.ONNX, "test/onnx-model")
    ]
)
@pytest.mark.unit
def test_valid_model_dir(model_type, model_dir):
    validate_model_dir(model_type, model_dir)



# Configuration file for the Sphinx documentation builder.
#
# This file only contains a selection of the most common options. For a full
# list see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Path setup --------------------------------------------------------------

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
import os
import sys

sys.path.append(os.path.abspath('../'))
sys.path.append(os.path.abspath('sphinxcontrib'))

# -- Project information -----------------------------------------------------

project = 'Merlin'
copyright = '2019, Gojek'
author = 'DSP Gojek'


# -- General configuration ---------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    'sphinx_rtd_theme',
    'sphinx.ext.autodoc',
    'sphinx.ext.doctest',
    'sphinx.ext.intersphinx',
    'sphinx.ext.todo',
    'sphinx.ext.coverage',
    'sphinx.ext.mathjax',
    'sphinx.ext.ifconfig',
    'sphinx.ext.viewcode',
    'sphinx.ext.napoleon',
    'sphinxcontrib.apidoc',
    'sphinxcontrib.confluencebuilder',
    'confluencebuilder_nbsphinx',
    'nbsphinx',
    'nbsphinx_link',  # for linking notebooks from outside sphinx source root
]

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = []


# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
html_theme = 'sphinx_rtd_theme'

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ['_static']

nbsphinx_execute = 'never'

# The master toctree document.
master_doc = 'index'

apidoc_module_dir = '../merlin'
apidoc_output_dir = 'source'
apidoc_excluded_paths = ['**/*test*']
apidoc_module_first = True

# Napoleon settings
napoleon_google_docstring = True
napoleon_numpy_docstring = True
napoleon_include_init_with_doc = True
napoleon_include_private_with_doc = False
napoleon_include_special_with_doc = True
napoleon_use_admonition_for_examples = False
napoleon_use_admonition_for_notes = False
napoleon_use_admonition_for_references = False
napoleon_use_ivar = False
napoleon_use_param = True
napoleon_use_rtype = False


confluence_publish = True
confluence_space_name = 'DSP'
confluence_parent_page = 'Merlin User Guide'
confluence_server_url = os.environ.get('CONFLUENCE_URL')
confluence_server_user = os.environ.get('CONFLUENCE_USER')
confluence_server_pass = os.environ.get('CONFLUENCE_PASSWORD')



# coding: utf-8

"""
    Merlin

    API Guide for accessing Merlin's model management, deployment, and serving functionalities  # noqa: E501

    OpenAPI spec version: 0.14.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class Environment(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'id': 'int',
        'name': 'str',
        'cluster': 'str',
        'is_default': 'bool',
        'region': 'str',
        'gcp_project': 'str',
        'default_resource_request': 'ResourceRequest',
        'default_transformer_resource_request': 'ResourceRequest',
        'default_prediction_job_resource_request': 'PredictionJobResourceRequest',
        'gpus': 'list[GPUConfig]',
        'created_at': 'datetime',
        'updated_at': 'datetime'
    }

    attribute_map = {
        'id': 'id',
        'name': 'name',
        'cluster': 'cluster',
        'is_default': 'is_default',
        'region': 'region',
        'gcp_project': 'gcp_project',
        'default_resource_request': 'default_resource_request',
        'default_transformer_resource_request': 'default_transformer_resource_request',
        'default_prediction_job_resource_request': 'default_prediction_job_resource_request',
        'gpus': 'gpus',
        'created_at': 'created_at',
        'updated_at': 'updated_at'
    }

    def __init__(self, id=None, name=None, cluster=None, is_default=None, region=None, gcp_project=None, default_resource_request=None, default_transformer_resource_request=None, default_prediction_job_resource_request=None, gpus=None, created_at=None, updated_at=None):  # noqa: E501
        """Environment - a model defined in Swagger"""  # noqa: E501
        self._id = None
        self._name = None
        self._cluster = None
        self._is_default = None
        self._region = None
        self._gcp_project = None
        self._default_resource_request = None
        self._default_transformer_resource_request = None
        self._default_prediction_job_resource_request = None
        self._gpus = None
        self._created_at = None
        self._updated_at = None
        self.discriminator = None
        if id is not None:
            self.id = id
        self.name = name
        if cluster is not None:
            self.cluster = cluster
        if is_default is not None:
            self.is_default = is_default
        if region is not None:
            self.region = region
        if gcp_project is not None:
            self.gcp_project = gcp_project
        if default_resource_request is not None:
            self.default_resource_request = default_resource_request
        if default_transformer_resource_request is not None:
            self.default_transformer_resource_request = default_transformer_resource_request
        if default_prediction_job_resource_request is not None:
            self.default_prediction_job_resource_request = default_prediction_job_resource_request
        if gpus is not None:
            self.gpus = gpus
        if created_at is not None:
            self.created_at = created_at
        if updated_at is not None:
            self.updated_at = updated_at

    @property
    def id(self):
        """Gets the id of this Environment.  # noqa: E501


        :return: The id of this Environment.  # noqa: E501
        :rtype: int
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this Environment.


        :param id: The id of this Environment.  # noqa: E501
        :type: int
        """

        self._id = id

    @property
    def name(self):
        """Gets the name of this Environment.  # noqa: E501


        :return: The name of this Environment.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this Environment.


        :param name: The name of this Environment.  # noqa: E501
        :type: str
        """
        if name is None:
            raise ValueError("Invalid value for `name`, must not be `None`")  # noqa: E501

        self._name = name

    @property
    def cluster(self):
        """Gets the cluster of this Environment.  # noqa: E501


        :return: The cluster of this Environment.  # noqa: E501
        :rtype: str
        """
        return self._cluster

    @cluster.setter
    def cluster(self, cluster):
        """Sets the cluster of this Environment.


        :param cluster: The cluster of this Environment.  # noqa: E501
        :type: str
        """

        self._cluster = cluster

    @property
    def is_default(self):
        """Gets the is_default of this Environment.  # noqa: E501


        :return: The is_default of this Environment.  # noqa: E501
        :rtype: bool
        """
        return self._is_default

    @is_default.setter
    def is_default(self, is_default):
        """Sets the is_default of this Environment.


        :param is_default: The is_default of this Environment.  # noqa: E501
        :type: bool
        """

        self._is_default = is_default

    @property
    def region(self):
        """Gets the region of this Environment.  # noqa: E501


        :return: The region of this Environment.  # noqa: E501
        :rtype: str
        """
        return self._region

    @region.setter
    def region(self, region):
        """Sets the region of this Environment.


        :param region: The region of this Environment.  # noqa: E501
        :type: str
        """

        self._region = region

    @property
    def gcp_project(self):
        """Gets the gcp_project of this Environment.  # noqa: E501


        :return: The gcp_project of this Environment.  # noqa: E501
        :rtype: str
        """
        return self._gcp_project

    @gcp_project.setter
    def gcp_project(self, gcp_project):
        """Sets the gcp_project of this Environment.


        :param gcp_project: The gcp_project of this Environment.  # noqa: E501
        :type: str
        """

        self._gcp_project = gcp_project

    @property
    def default_resource_request(self):
        """Gets the default_resource_request of this Environment.  # noqa: E501


        :return: The default_resource_request of this Environment.  # noqa: E501
        :rtype: ResourceRequest
        """
        return self._default_resource_request

    @default_resource_request.setter
    def default_resource_request(self, default_resource_request):
        """Sets the default_resource_request of this Environment.


        :param default_resource_request: The default_resource_request of this Environment.  # noqa: E501
        :type: ResourceRequest
        """

        self._default_resource_request = default_resource_request

    @property
    def default_transformer_resource_request(self):
        """Gets the default_transformer_resource_request of this Environment.  # noqa: E501


        :return: The default_transformer_resource_request of this Environment.  # noqa: E501
        :rtype: ResourceRequest
        """
        return self._default_transformer_resource_request

    @default_transformer_resource_request.setter
    def default_transformer_resource_request(self, default_transformer_resource_request):
        """Sets the default_transformer_resource_request of this Environment.


        :param default_transformer_resource_request: The default_transformer_resource_request of this Environment.  # noqa: E501
        :type: ResourceRequest
        """

        self._default_transformer_resource_request = default_transformer_resource_request

    @property
    def default_prediction_job_resource_request(self):
        """Gets the default_prediction_job_resource_request of this Environment.  # noqa: E501


        :return: The default_prediction_job_resource_request of this Environment.  # noqa: E501
        :rtype: PredictionJobResourceRequest
        """
        return self._default_prediction_job_resource_request

    @default_prediction_job_resource_request.setter
    def default_prediction_job_resource_request(self, default_prediction_job_resource_request):
        """Sets the default_prediction_job_resource_request of this Environment.


        :param default_prediction_job_resource_request: The default_prediction_job_resource_request of this Environment.  # noqa: E501
        :type: PredictionJobResourceRequest
        """

        self._default_prediction_job_resource_request = default_prediction_job_resource_request

    @property
    def gpus(self):
        """Gets the gpus of this Environment.  # noqa: E501


        :return: The gpus of this Environment.  # noqa: E501
        :rtype: list[GPUConfig]
        """
        return self._gpus

    @gpus.setter
    def gpus(self, gpus):
        """Sets the gpus of this Environment.


        :param gpus: The gpus of this Environment.  # noqa: E501
        :type: list[GPUConfig]
        """

        self._gpus = gpus

    @property
    def created_at(self):
        """Gets the created_at of this Environment.  # noqa: E501


        :return: The created_at of this Environment.  # noqa: E501
        :rtype: datetime
        """
        return self._created_at

    @created_at.setter
    def created_at(self, created_at):
        """Sets the created_at of this Environment.


        :param created_at: The created_at of this Environment.  # noqa: E501
        :type: datetime
        """

        self._created_at = created_at

    @property
    def updated_at(self):
        """Gets the updated_at of this Environment.  # noqa: E501


        :return: The updated_at of this Environment.  # noqa: E501
        :rtype: datetime
        """
        return self._updated_at

    @updated_at.setter
    def updated_at(self, updated_at):
        """Sets the updated_at of this Environment.


        :param updated_at: The updated_at of this Environment.  # noqa: E501
        :type: datetime
        """

        self._updated_at = updated_at

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(Environment, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, Environment):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Merlin

    API Guide for accessing Merlin's model management, deployment, and serving functionalities  # noqa: E501

    OpenAPI spec version: 0.14.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class GPUConfig(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'name': 'str',
        'values': 'list[str]',
        'resource_type': 'str',
        'node_selector': 'dict(str, str)',
        'tolerations': 'list[GPUToleration]',
        'min_monthly_cost_per_gpu': 'float',
        'max_monthly_cost_per_gpu': 'float'
    }

    attribute_map = {
        'name': 'name',
        'values': 'values',
        'resource_type': 'resource_type',
        'node_selector': 'node_selector',
        'tolerations': 'tolerations',
        'min_monthly_cost_per_gpu': 'min_monthly_cost_per_gpu',
        'max_monthly_cost_per_gpu': 'max_monthly_cost_per_gpu'
    }

    def __init__(self, name=None, values=None, resource_type=None, node_selector=None, tolerations=None, min_monthly_cost_per_gpu=None, max_monthly_cost_per_gpu=None):  # noqa: E501
        """GPUConfig - a model defined in Swagger"""  # noqa: E501
        self._name = None
        self._values = None
        self._resource_type = None
        self._node_selector = None
        self._tolerations = None
        self._min_monthly_cost_per_gpu = None
        self._max_monthly_cost_per_gpu = None
        self.discriminator = None
        if name is not None:
            self.name = name
        if values is not None:
            self.values = values
        if resource_type is not None:
            self.resource_type = resource_type
        if node_selector is not None:
            self.node_selector = node_selector
        if tolerations is not None:
            self.tolerations = tolerations
        if min_monthly_cost_per_gpu is not None:
            self.min_monthly_cost_per_gpu = min_monthly_cost_per_gpu
        if max_monthly_cost_per_gpu is not None:
            self.max_monthly_cost_per_gpu = max_monthly_cost_per_gpu

    @property
    def name(self):
        """Gets the name of this GPUConfig.  # noqa: E501


        :return: The name of this GPUConfig.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this GPUConfig.


        :param name: The name of this GPUConfig.  # noqa: E501
        :type: str
        """

        self._name = name

    @property
    def values(self):
        """Gets the values of this GPUConfig.  # noqa: E501


        :return: The values of this GPUConfig.  # noqa: E501
        :rtype: list[str]
        """
        return self._values

    @values.setter
    def values(self, values):
        """Sets the values of this GPUConfig.


        :param values: The values of this GPUConfig.  # noqa: E501
        :type: list[str]
        """

        self._values = values

    @property
    def resource_type(self):
        """Gets the resource_type of this GPUConfig.  # noqa: E501


        :return: The resource_type of this GPUConfig.  # noqa: E501
        :rtype: str
        """
        return self._resource_type

    @resource_type.setter
    def resource_type(self, resource_type):
        """Sets the resource_type of this GPUConfig.


        :param resource_type: The resource_type of this GPUConfig.  # noqa: E501
        :type: str
        """

        self._resource_type = resource_type

    @property
    def node_selector(self):
        """Gets the node_selector of this GPUConfig.  # noqa: E501


        :return: The node_selector of this GPUConfig.  # noqa: E501
        :rtype: dict(str, str)
        """
        return self._node_selector

    @node_selector.setter
    def node_selector(self, node_selector):
        """Sets the node_selector of this GPUConfig.


        :param node_selector: The node_selector of this GPUConfig.  # noqa: E501
        :type: dict(str, str)
        """

        self._node_selector = node_selector

    @property
    def tolerations(self):
        """Gets the tolerations of this GPUConfig.  # noqa: E501


        :return: The tolerations of this GPUConfig.  # noqa: E501
        :rtype: list[GPUToleration]
        """
        return self._tolerations

    @tolerations.setter
    def tolerations(self, tolerations):
        """Sets the tolerations of this GPUConfig.


        :param tolerations: The tolerations of this GPUConfig.  # noqa: E501
        :type: list[GPUToleration]
        """

        self._tolerations = tolerations

    @property
    def min_monthly_cost_per_gpu(self):
        """Gets the min_monthly_cost_per_gpu of this GPUConfig.  # noqa: E501


        :return: The min_monthly_cost_per_gpu of this GPUConfig.  # noqa: E501
        :rtype: float
        """
        return self._min_monthly_cost_per_gpu

    @min_monthly_cost_per_gpu.setter
    def min_monthly_cost_per_gpu(self, min_monthly_cost_per_gpu):
        """Sets the min_monthly_cost_per_gpu of this GPUConfig.


        :param min_monthly_cost_per_gpu: The min_monthly_cost_per_gpu of this GPUConfig.  # noqa: E501
        :type: float
        """

        self._min_monthly_cost_per_gpu = min_monthly_cost_per_gpu

    @property
    def max_monthly_cost_per_gpu(self):
        """Gets the max_monthly_cost_per_gpu of this GPUConfig.  # noqa: E501


        :return: The max_monthly_cost_per_gpu of this GPUConfig.  # noqa: E501
        :rtype: float
        """
        return self._max_monthly_cost_per_gpu

    @max_monthly_cost_per_gpu.setter
    def max_monthly_cost_per_gpu(self, max_monthly_cost_per_gpu):
        """Sets the max_monthly_cost_per_gpu of this GPUConfig.


        :param max_monthly_cost_per_gpu: The max_monthly_cost_per_gpu of this GPUConfig.  # noqa: E501
        :type: float
        """

        self._max_monthly_cost_per_gpu = max_monthly_cost_per_gpu

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(GPUConfig, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, GPUConfig):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Merlin

    API Guide for accessing Merlin's model management, deployment, and serving functionalities  # noqa: E501

    OpenAPI spec version: 0.14.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

from __future__ import absolute_import

import re  # noqa: F401

# python 2 and python 3 compatibility library
import six

from client.api_client import ApiClient


class LogApi(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    Ref: https://github.com/swagger-api/swagger-codegen
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

    def logs_get(self, cluster, namespace, component_type, **kwargs):  # noqa: E501
        """Retrieve log from a container  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.logs_get(cluster, namespace, component_type, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str cluster: (required)
        :param str namespace: (required)
        :param str component_type: (required)
        :param str project_name:
        :param str model_id:
        :param str model_name:
        :param str version_id:
        :param str prediction_job_id:
        :param str container_name:
        :param str prefix:
        :param str follow:
        :param str previous:
        :param str since_seconds:
        :param str since_time:
        :param str timestamps:
        :param str tail_lines:
        :param str limit_bytes:
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.logs_get_with_http_info(cluster, namespace, component_type, **kwargs)  # noqa: E501
        else:
            (data) = self.logs_get_with_http_info(cluster, namespace, component_type, **kwargs)  # noqa: E501
            return data

    def logs_get_with_http_info(self, cluster, namespace, component_type, **kwargs):  # noqa: E501
        """Retrieve log from a container  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.logs_get_with_http_info(cluster, namespace, component_type, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str cluster: (required)
        :param str namespace: (required)
        :param str component_type: (required)
        :param str project_name:
        :param str model_id:
        :param str model_name:
        :param str version_id:
        :param str prediction_job_id:
        :param str container_name:
        :param str prefix:
        :param str follow:
        :param str previous:
        :param str since_seconds:
        :param str since_time:
        :param str timestamps:
        :param str tail_lines:
        :param str limit_bytes:
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['cluster', 'namespace', 'component_type', 'project_name', 'model_id', 'model_name', 'version_id', 'prediction_job_id', 'container_name', 'prefix', 'follow', 'previous', 'since_seconds', 'since_time', 'timestamps', 'tail_lines', 'limit_bytes']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method logs_get" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'cluster' is set
        if ('cluster' not in params or
                params['cluster'] is None):
            raise ValueError("Missing the required parameter `cluster` when calling `logs_get`")  # noqa: E501
        # verify the required parameter 'namespace' is set
        if ('namespace' not in params or
                params['namespace'] is None):
            raise ValueError("Missing the required parameter `namespace` when calling `logs_get`")  # noqa: E501
        # verify the required parameter 'component_type' is set
        if ('component_type' not in params or
                params['component_type'] is None):
            raise ValueError("Missing the required parameter `component_type` when calling `logs_get`")  # noqa: E501

        collection_formats = {}

        path_params = {}

        query_params = []
        if 'project_name' in params:
            query_params.append(('project_name', params['project_name']))  # noqa: E501
        if 'model_id' in params:
            query_params.append(('model_id', params['model_id']))  # noqa: E501
        if 'model_name' in params:
            query_params.append(('model_name', params['model_name']))  # noqa: E501
        if 'version_id' in params:
            query_params.append(('version_id', params['version_id']))  # noqa: E501
        if 'prediction_job_id' in params:
            query_params.append(('prediction_job_id', params['prediction_job_id']))  # noqa: E501
        if 'cluster' in params:
            query_params.append(('cluster', params['cluster']))  # noqa: E501
        if 'namespace' in params:
            query_params.append(('namespace', params['namespace']))  # noqa: E501
        if 'component_type' in params:
            query_params.append(('component_type', params['component_type']))  # noqa: E501
        if 'container_name' in params:
            query_params.append(('container_name', params['container_name']))  # noqa: E501
        if 'prefix' in params:
            query_params.append(('prefix', params['prefix']))  # noqa: E501
        if 'follow' in params:
            query_params.append(('follow', params['follow']))  # noqa: E501
        if 'previous' in params:
            query_params.append(('previous', params['previous']))  # noqa: E501
        if 'since_seconds' in params:
            query_params.append(('since_seconds', params['since_seconds']))  # noqa: E501
        if 'since_time' in params:
            query_params.append(('since_time', params['since_time']))  # noqa: E501
        if 'timestamps' in params:
            query_params.append(('timestamps', params['timestamps']))  # noqa: E501
        if 'tail_lines' in params:
            query_params.append(('tail_lines', params['tail_lines']))  # noqa: E501
        if 'limit_bytes' in params:
            query_params.append(('limit_bytes', params['limit_bytes']))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # Authentication setting
        auth_settings = ['Bearer']  # noqa: E501

        return self.api_client.call_api(
            '/logs', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type=None,  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)



# Copyright 2020 The Merlin Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import pytest

from merlin.resource_request import ResourceRequest


@pytest.mark.unit
def test_resource_request_validate():
    resource_request = ResourceRequest(1, 2, "100m", "128Mi")
    resource_request.validate()

    resource_request.min_replica = 10
    with pytest.raises(Exception, match="Min replica must be less or equal to max replica"):
        resource_request.validate()



# Copyright 2020 The Merlin Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import pytest

from merlin.batch.source import BigQuerySource


@pytest.mark.parametrize(
    "table,features,options,expected_dict,expected_valid", [
        pytest.param(
            'project.dataset.table',
            ['driver_id', 'completed_orders'],
            {'project': 'project'},
            {
                'table': 'project.dataset.table',
                'features': ['driver_id', 'completed_orders'],
                'options': {'project': 'project'}
            },
            True,
            id="Should valid if source has valid_types and valid property values"
        ),
        pytest.param(
            'project.dataset.table',
            ['driver_id', 'completed_orders'],
            {},
            {
                'table': 'project.dataset.table',
                'features': ['driver_id', 'completed_orders'],
                'options': {}
            },
            True,
            id="Should valid if source has valid_types and valid property value(2)"
        ),
        pytest.param(
            'project.dataset.table',
            ['driver_id', 'completed_orders'],
            None,
            {
                'table': 'project.dataset.table',
                'features': ['driver_id', 'completed_orders'],
                'options': {}
            },
            True,
            id="Should valid if source has valid_types and valid property value(3)"
        ),
        pytest.param(
            'project.dataset',
            ['driver_id', 'completed_orders'],
            None,
            {},
            False,
            id="Should not valid and return empty dictionary if source table is not valid"
        ),
        pytest.param(
            1,
            ['driver_id', 'completed_orders'],
            None,
            {},
            False,
            id="Should not valid and return empty dictionary if source table doesn't have valid data type, it should be string"
        ),
        pytest.param(
            'project.dataset.table',
            [1, 2, 3],
            None,
            {},
            False,
            id="Should not valid and return empty dictionary if features doesn't have valid data type, it should be array of string"
        ),
        pytest.param(
            'project.dataset.table',
            'driver_id',
            None,
            {},
            False,
            id="Should not valid and return empty dictionary if features doesn't have valid data type, it should be array of string(2)"
        )
    ]
)
def test_dictionary_generated(table, features, options, expected_dict, expected_valid):
    biq_query_source = BigQuerySource(table, features, options)
    if expected_valid:
        assert biq_query_source.to_dict() == expected_dict
    else:
        with pytest.raises(ValueError):
            biq_query_source.to_dict()


@pytest.mark.parametrize(
    "table,features,options,expected", [
        pytest.param(
            'project.dataset.table', ['driver_id', 'completed_orders'], {'project': 'project'}, True,
            id="Should valid if source has valid_types and valid property value"
        ),
        pytest.param(
            'project.dataset.table', ['driver_id', 'completed_orders'], None, True,
            id="Should valid if source has valid_types and valid property value(2)"
        ),
        pytest.param(
            'project.dataset.table', ['driver_id', 'completed_orders'], ['project', 'environment'], False,
            id="Should not valid if options has wrong data type, it should be dictionary"
        ),
        pytest.param(
            'project', ['driver_id', 'completed_orders'], {'project': 'project'}, False,
            id="Should not valid if source_table is not valid"
        ),
        pytest.param(
            'project.dataset', ['driver_id', 'completed_orders'], {'project': 'project'}, False,
            id="Should not valid if source_table is not valid(2)"
        ),
        pytest.param(
            'project.dataset.table', [1, 2, 3], {'project': 'project'}, False,
            id="Should not valid if features doesn't have correct data types, it should be array of string"
        ),
        pytest.param(
            'project.dataset.table', 'driver_id', {'project': 'project'}, False,
            id="Should not valid if features doesn't have correct data types, it should be array of string"
        ),
        pytest.param(
            'project.dataset.table', 14, {'project': 'project'}, False,
            id="Should not valid, features doesn't have correct data types, it should be array of string"
        ),
        pytest.param(
            'project.dataset.table', ['driver_id', '_TABLE_driver'], {'project': 'project'}, False,
            id="Should not valid if features contains invalid column name"
        ),
    ]
)
def test_valid(table, features, options, expected):
    biq_query_source = BigQuerySource(table, features, options)
    if expected:
        biq_query_source._validate()
    else:
        with pytest.raises(ValueError):
            biq_query_source._validate()



# coding: utf-8

"""
    Merlin

    API Guide for accessing Merlin's model management, deployment, and serving functionalities  # noqa: E501

    OpenAPI spec version: 0.14.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class ModelEndpointAlert(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'model_id': 'int',
        'model_endpoint_id': 'int',
        'environment_name': 'str',
        'team_name': 'str',
        'alert_conditions': 'list[ModelEndpointAlertCondition]'
    }

    attribute_map = {
        'model_id': 'model_id',
        'model_endpoint_id': 'model_endpoint_id',
        'environment_name': 'environment_name',
        'team_name': 'team_name',
        'alert_conditions': 'alert_conditions'
    }

    def __init__(self, model_id=None, model_endpoint_id=None, environment_name=None, team_name=None, alert_conditions=None):  # noqa: E501
        """ModelEndpointAlert - a model defined in Swagger"""  # noqa: E501
        self._model_id = None
        self._model_endpoint_id = None
        self._environment_name = None
        self._team_name = None
        self._alert_conditions = None
        self.discriminator = None
        if model_id is not None:
            self.model_id = model_id
        if model_endpoint_id is not None:
            self.model_endpoint_id = model_endpoint_id
        if environment_name is not None:
            self.environment_name = environment_name
        if team_name is not None:
            self.team_name = team_name
        if alert_conditions is not None:
            self.alert_conditions = alert_conditions

    @property
    def model_id(self):
        """Gets the model_id of this ModelEndpointAlert.  # noqa: E501


        :return: The model_id of this ModelEndpointAlert.  # noqa: E501
        :rtype: int
        """
        return self._model_id

    @model_id.setter
    def model_id(self, model_id):
        """Sets the model_id of this ModelEndpointAlert.


        :param model_id: The model_id of this ModelEndpointAlert.  # noqa: E501
        :type: int
        """

        self._model_id = model_id

    @property
    def model_endpoint_id(self):
        """Gets the model_endpoint_id of this ModelEndpointAlert.  # noqa: E501


        :return: The model_endpoint_id of this ModelEndpointAlert.  # noqa: E501
        :rtype: int
        """
        return self._model_endpoint_id

    @model_endpoint_id.setter
    def model_endpoint_id(self, model_endpoint_id):
        """Sets the model_endpoint_id of this ModelEndpointAlert.


        :param model_endpoint_id: The model_endpoint_id of this ModelEndpointAlert.  # noqa: E501
        :type: int
        """

        self._model_endpoint_id = model_endpoint_id

    @property
    def environment_name(self):
        """Gets the environment_name of this ModelEndpointAlert.  # noqa: E501


        :return: The environment_name of this ModelEndpointAlert.  # noqa: E501
        :rtype: str
        """
        return self._environment_name

    @environment_name.setter
    def environment_name(self, environment_name):
        """Sets the environment_name of this ModelEndpointAlert.


        :param environment_name: The environment_name of this ModelEndpointAlert.  # noqa: E501
        :type: str
        """

        self._environment_name = environment_name

    @property
    def team_name(self):
        """Gets the team_name of this ModelEndpointAlert.  # noqa: E501


        :return: The team_name of this ModelEndpointAlert.  # noqa: E501
        :rtype: str
        """
        return self._team_name

    @team_name.setter
    def team_name(self, team_name):
        """Sets the team_name of this ModelEndpointAlert.


        :param team_name: The team_name of this ModelEndpointAlert.  # noqa: E501
        :type: str
        """

        self._team_name = team_name

    @property
    def alert_conditions(self):
        """Gets the alert_conditions of this ModelEndpointAlert.  # noqa: E501


        :return: The alert_conditions of this ModelEndpointAlert.  # noqa: E501
        :rtype: list[ModelEndpointAlertCondition]
        """
        return self._alert_conditions

    @alert_conditions.setter
    def alert_conditions(self, alert_conditions):
        """Sets the alert_conditions of this ModelEndpointAlert.


        :param alert_conditions: The alert_conditions of this ModelEndpointAlert.  # noqa: E501
        :type: list[ModelEndpointAlertCondition]
        """

        self._alert_conditions = alert_conditions

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(ModelEndpointAlert, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ModelEndpointAlert):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Merlin

    API Guide for accessing Merlin's model management, deployment, and serving functionalities  # noqa: E501

    OpenAPI spec version: 0.14.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class PredictionJobConfig(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'version': 'str',
        'kind': 'str',
        'name': 'str',
        'bigquery_source': 'PredictionJobConfigBigquerySource',
        'gcs_source': 'PredictionJobConfigGcsSource',
        'model': 'PredictionJobConfigModel',
        'bigquery_sink': 'PredictionJobConfigBigquerySink',
        'gcs_sink': 'PredictionJobConfigGcsSink'
    }

    attribute_map = {
        'version': 'version',
        'kind': 'kind',
        'name': 'name',
        'bigquery_source': 'bigquery_source',
        'gcs_source': 'gcs_source',
        'model': 'model',
        'bigquery_sink': 'bigquery_sink',
        'gcs_sink': 'gcs_sink'
    }

    def __init__(self, version=None, kind=None, name=None, bigquery_source=None, gcs_source=None, model=None, bigquery_sink=None, gcs_sink=None):  # noqa: E501
        """PredictionJobConfig - a model defined in Swagger"""  # noqa: E501
        self._version = None
        self._kind = None
        self._name = None
        self._bigquery_source = None
        self._gcs_source = None
        self._model = None
        self._bigquery_sink = None
        self._gcs_sink = None
        self.discriminator = None
        if version is not None:
            self.version = version
        if kind is not None:
            self.kind = kind
        if name is not None:
            self.name = name
        if bigquery_source is not None:
            self.bigquery_source = bigquery_source
        if gcs_source is not None:
            self.gcs_source = gcs_source
        if model is not None:
            self.model = model
        if bigquery_sink is not None:
            self.bigquery_sink = bigquery_sink
        if gcs_sink is not None:
            self.gcs_sink = gcs_sink

    @property
    def version(self):
        """Gets the version of this PredictionJobConfig.  # noqa: E501


        :return: The version of this PredictionJobConfig.  # noqa: E501
        :rtype: str
        """
        return self._version

    @version.setter
    def version(self, version):
        """Sets the version of this PredictionJobConfig.


        :param version: The version of this PredictionJobConfig.  # noqa: E501
        :type: str
        """

        self._version = version

    @property
    def kind(self):
        """Gets the kind of this PredictionJobConfig.  # noqa: E501


        :return: The kind of this PredictionJobConfig.  # noqa: E501
        :rtype: str
        """
        return self._kind

    @kind.setter
    def kind(self, kind):
        """Sets the kind of this PredictionJobConfig.


        :param kind: The kind of this PredictionJobConfig.  # noqa: E501
        :type: str
        """

        self._kind = kind

    @property
    def name(self):
        """Gets the name of this PredictionJobConfig.  # noqa: E501


        :return: The name of this PredictionJobConfig.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this PredictionJobConfig.


        :param name: The name of this PredictionJobConfig.  # noqa: E501
        :type: str
        """

        self._name = name

    @property
    def bigquery_source(self):
        """Gets the bigquery_source of this PredictionJobConfig.  # noqa: E501


        :return: The bigquery_source of this PredictionJobConfig.  # noqa: E501
        :rtype: PredictionJobConfigBigquerySource
        """
        return self._bigquery_source

    @bigquery_source.setter
    def bigquery_source(self, bigquery_source):
        """Sets the bigquery_source of this PredictionJobConfig.


        :param bigquery_source: The bigquery_source of this PredictionJobConfig.  # noqa: E501
        :type: PredictionJobConfigBigquerySource
        """

        self._bigquery_source = bigquery_source

    @property
    def gcs_source(self):
        """Gets the gcs_source of this PredictionJobConfig.  # noqa: E501


        :return: The gcs_source of this PredictionJobConfig.  # noqa: E501
        :rtype: PredictionJobConfigGcsSource
        """
        return self._gcs_source

    @gcs_source.setter
    def gcs_source(self, gcs_source):
        """Sets the gcs_source of this PredictionJobConfig.


        :param gcs_source: The gcs_source of this PredictionJobConfig.  # noqa: E501
        :type: PredictionJobConfigGcsSource
        """

        self._gcs_source = gcs_source

    @property
    def model(self):
        """Gets the model of this PredictionJobConfig.  # noqa: E501


        :return: The model of this PredictionJobConfig.  # noqa: E501
        :rtype: PredictionJobConfigModel
        """
        return self._model

    @model.setter
    def model(self, model):
        """Sets the model of this PredictionJobConfig.


        :param model: The model of this PredictionJobConfig.  # noqa: E501
        :type: PredictionJobConfigModel
        """

        self._model = model

    @property
    def bigquery_sink(self):
        """Gets the bigquery_sink of this PredictionJobConfig.  # noqa: E501


        :return: The bigquery_sink of this PredictionJobConfig.  # noqa: E501
        :rtype: PredictionJobConfigBigquerySink
        """
        return self._bigquery_sink

    @bigquery_sink.setter
    def bigquery_sink(self, bigquery_sink):
        """Sets the bigquery_sink of this PredictionJobConfig.


        :param bigquery_sink: The bigquery_sink of this PredictionJobConfig.  # noqa: E501
        :type: PredictionJobConfigBigquerySink
        """

        self._bigquery_sink = bigquery_sink

    @property
    def gcs_sink(self):
        """Gets the gcs_sink of this PredictionJobConfig.  # noqa: E501


        :return: The gcs_sink of this PredictionJobConfig.  # noqa: E501
        :rtype: PredictionJobConfigGcsSink
        """
        return self._gcs_sink

    @gcs_sink.setter
    def gcs_sink(self, gcs_sink):
        """Sets the gcs_sink of this PredictionJobConfig.


        :param gcs_sink: The gcs_sink of this PredictionJobConfig.  # noqa: E501
        :type: PredictionJobConfigGcsSink
        """

        self._gcs_sink = gcs_sink

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(PredictionJobConfig, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, PredictionJobConfig):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



from unittest.mock import MagicMock

import grpc
from caraml.upi.v1 import upi_pb2
from mlflow.pyfunc import PythonModelContext


from merlin.model import PyFuncModel, PYFUNC_MODEL_INPUT_KEY, PYFUNC_EXTRA_ARGS_KEY
from merlin.protocol import Protocol
from merlin.pyfunc import PYFUNC_PROTOCOL_KEY, PYFUNC_GRPC_CONTEXT

request = {
    "instances": [[1, 2, 3, 4]]
}

response = {
    "predictions": [[1]]
}

headers = {
    "key" : "value"
}

upiv1_request = upi_pb2.PredictValuesRequest()
upiv1_response = upi_pb2.PredictValuesResponse()
grpc_context = {}

def test_http():
    pyfunc_model = PyFuncModel()
    pyfunc_model.infer = MagicMock(return_value=response)

    context = PythonModelContext(artifacts = {})

    pyfunc_model.load_context(context)
    model_input = {
        PYFUNC_MODEL_INPUT_KEY: request,
    }

    actual_resp = pyfunc_model.predict(context=context, model_input=model_input)
    assert actual_resp == response
    pyfunc_model.infer.assert_called_with(request)


def test_http_headers():
    pyfunc_model = PyFuncModel()
    pyfunc_model.infer = MagicMock(return_value=response)

    context = PythonModelContext(artifacts = {})

    pyfunc_model.load_context(context)
    model_input = {
        PYFUNC_MODEL_INPUT_KEY: request,
        PYFUNC_EXTRA_ARGS_KEY: {
            "headers" : headers
        }
    }

    actual_resp = pyfunc_model.predict(context=context, model_input=model_input)
    assert actual_resp == response
    pyfunc_model.infer.assert_called_with(request, headers=headers)


def test_explicit_protocol():
    pyfunc_model = PyFuncModel()
    pyfunc_model.infer = MagicMock(return_value=response)

    context = PythonModelContext(artifacts = {})

    pyfunc_model.load_context(context)
    model_input = {
        PYFUNC_MODEL_INPUT_KEY: request,
        PYFUNC_PROTOCOL_KEY: Protocol.HTTP_JSON
    }

    actual_resp = pyfunc_model.predict(context=context, model_input=model_input)
    assert actual_resp == response
    pyfunc_model.infer.assert_called_with(request)


def test_upiv1():
    pyfunc_model = PyFuncModel()
    pyfunc_model.upiv1_infer = MagicMock(return_value=upiv1_response)

    context = PythonModelContext(artifacts = {})

    pyfunc_model.load_context(context)
    model_input = {
        PYFUNC_PROTOCOL_KEY: Protocol.UPI_V1,
        PYFUNC_MODEL_INPUT_KEY: upiv1_request,
        PYFUNC_GRPC_CONTEXT: grpc_context,
    }

    actual_resp = pyfunc_model.predict(context=context, model_input=model_input)
    assert actual_resp == upiv1_response
    pyfunc_model.upiv1_infer.assert_called_with(upiv1_request, grpc_context)


# coding: utf-8

"""
    Merlin

    API Guide for accessing Merlin's model management, deployment, and serving functionalities  # noqa: E501

    OpenAPI spec version: 0.14.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

from __future__ import absolute_import

import re  # noqa: F401

# python 2 and python 3 compatibility library
import six

from client.api_client import ApiClient


class ModelEndpointsApi(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    Ref: https://github.com/swagger-api/swagger-codegen
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

    def models_model_id_endpoints_get(self, model_id, **kwargs):  # noqa: E501
        """List model endpoint  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.models_model_id_endpoints_get(model_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param int model_id: (required)
        :return: list[ModelEndpoint]
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.models_model_id_endpoints_get_with_http_info(model_id, **kwargs)  # noqa: E501
        else:
            (data) = self.models_model_id_endpoints_get_with_http_info(model_id, **kwargs)  # noqa: E501
            return data

    def models_model_id_endpoints_get_with_http_info(self, model_id, **kwargs):  # noqa: E501
        """List model endpoint  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.models_model_id_endpoints_get_with_http_info(model_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param int model_id: (required)
        :return: list[ModelEndpoint]
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['model_id']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method models_model_id_endpoints_get" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'model_id' is set
        if ('model_id' not in params or
                params['model_id'] is None):
            raise ValueError("Missing the required parameter `model_id` when calling `models_model_id_endpoints_get`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'model_id' in params:
            path_params['model_id'] = params['model_id']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['*/*'])  # noqa: E501

        # Authentication setting
        auth_settings = ['Bearer']  # noqa: E501

        return self.api_client.call_api(
            '/models/{model_id}/endpoints', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='list[ModelEndpoint]',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def models_model_id_endpoints_model_endpoint_id_delete(self, model_id, model_endpoint_id, **kwargs):  # noqa: E501
        """Stop serving traffic to the model endpoint, then delete it.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.models_model_id_endpoints_model_endpoint_id_delete(model_id, model_endpoint_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param int model_id: (required)
        :param str model_endpoint_id: (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.models_model_id_endpoints_model_endpoint_id_delete_with_http_info(model_id, model_endpoint_id, **kwargs)  # noqa: E501
        else:
            (data) = self.models_model_id_endpoints_model_endpoint_id_delete_with_http_info(model_id, model_endpoint_id, **kwargs)  # noqa: E501
            return data

    def models_model_id_endpoints_model_endpoint_id_delete_with_http_info(self, model_id, model_endpoint_id, **kwargs):  # noqa: E501
        """Stop serving traffic to the model endpoint, then delete it.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.models_model_id_endpoints_model_endpoint_id_delete_with_http_info(model_id, model_endpoint_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param int model_id: (required)
        :param str model_endpoint_id: (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['model_id', 'model_endpoint_id']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method models_model_id_endpoints_model_endpoint_id_delete" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'model_id' is set
        if ('model_id' not in params or
                params['model_id'] is None):
            raise ValueError("Missing the required parameter `model_id` when calling `models_model_id_endpoints_model_endpoint_id_delete`")  # noqa: E501
        # verify the required parameter 'model_endpoint_id' is set
        if ('model_endpoint_id' not in params or
                params['model_endpoint_id'] is None):
            raise ValueError("Missing the required parameter `model_endpoint_id` when calling `models_model_id_endpoints_model_endpoint_id_delete`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'model_id' in params:
            path_params['model_id'] = params['model_id']  # noqa: E501
        if 'model_endpoint_id' in params:
            path_params['model_endpoint_id'] = params['model_endpoint_id']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # Authentication setting
        auth_settings = ['Bearer']  # noqa: E501

        return self.api_client.call_api(
            '/models/{model_id}/endpoints/{model_endpoint_id}', 'DELETE',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type=None,  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def models_model_id_endpoints_model_endpoint_id_get(self, model_id, model_endpoint_id, **kwargs):  # noqa: E501
        """Get a model endpoint  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.models_model_id_endpoints_model_endpoint_id_get(model_id, model_endpoint_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param int model_id: (required)
        :param str model_endpoint_id: (required)
        :return: ModelEndpoint
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.models_model_id_endpoints_model_endpoint_id_get_with_http_info(model_id, model_endpoint_id, **kwargs)  # noqa: E501
        else:
            (data) = self.models_model_id_endpoints_model_endpoint_id_get_with_http_info(model_id, model_endpoint_id, **kwargs)  # noqa: E501
            return data

    def models_model_id_endpoints_model_endpoint_id_get_with_http_info(self, model_id, model_endpoint_id, **kwargs):  # noqa: E501
        """Get a model endpoint  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.models_model_id_endpoints_model_endpoint_id_get_with_http_info(model_id, model_endpoint_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param int model_id: (required)
        :param str model_endpoint_id: (required)
        :return: ModelEndpoint
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['model_id', 'model_endpoint_id']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method models_model_id_endpoints_model_endpoint_id_get" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'model_id' is set
        if ('model_id' not in params or
                params['model_id'] is None):
            raise ValueError("Missing the required parameter `model_id` when calling `models_model_id_endpoints_model_endpoint_id_get`")  # noqa: E501
        # verify the required parameter 'model_endpoint_id' is set
        if ('model_endpoint_id' not in params or
                params['model_endpoint_id'] is None):
            raise ValueError("Missing the required parameter `model_endpoint_id` when calling `models_model_id_endpoints_model_endpoint_id_get`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'model_id' in params:
            path_params['model_id'] = params['model_id']  # noqa: E501
        if 'model_endpoint_id' in params:
            path_params['model_endpoint_id'] = params['model_endpoint_id']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['*/*'])  # noqa: E501

        # Authentication setting
        auth_settings = ['Bearer']  # noqa: E501

        return self.api_client.call_api(
            '/models/{model_id}/endpoints/{model_endpoint_id}', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='ModelEndpoint',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def models_model_id_endpoints_model_endpoint_id_put(self, model_id, model_endpoint_id, **kwargs):  # noqa: E501
        """Update model endpoint data. Mainly used to update its rule.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.models_model_id_endpoints_model_endpoint_id_put(model_id, model_endpoint_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param int model_id: (required)
        :param str model_endpoint_id: (required)
        :param ModelEndpoint body:
        :return: ModelEndpoint
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.models_model_id_endpoints_model_endpoint_id_put_with_http_info(model_id, model_endpoint_id, **kwargs)  # noqa: E501
        else:
            (data) = self.models_model_id_endpoints_model_endpoint_id_put_with_http_info(model_id, model_endpoint_id, **kwargs)  # noqa: E501
            return data

    def models_model_id_endpoints_model_endpoint_id_put_with_http_info(self, model_id, model_endpoint_id, **kwargs):  # noqa: E501
        """Update model endpoint data. Mainly used to update its rule.  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.models_model_id_endpoints_model_endpoint_id_put_with_http_info(model_id, model_endpoint_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param int model_id: (required)
        :param str model_endpoint_id: (required)
        :param ModelEndpoint body:
        :return: ModelEndpoint
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['model_id', 'model_endpoint_id', 'body']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method models_model_id_endpoints_model_endpoint_id_put" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'model_id' is set
        if ('model_id' not in params or
                params['model_id'] is None):
            raise ValueError("Missing the required parameter `model_id` when calling `models_model_id_endpoints_model_endpoint_id_put`")  # noqa: E501
        # verify the required parameter 'model_endpoint_id' is set
        if ('model_endpoint_id' not in params or
                params['model_endpoint_id'] is None):
            raise ValueError("Missing the required parameter `model_endpoint_id` when calling `models_model_id_endpoints_model_endpoint_id_put`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'model_id' in params:
            path_params['model_id'] = params['model_id']  # noqa: E501
        if 'model_endpoint_id' in params:
            path_params['model_endpoint_id'] = params['model_endpoint_id']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'body' in params:
            body_params = params['body']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['*/*'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['*/*'])  # noqa: E501

        # Authentication setting
        auth_settings = ['Bearer']  # noqa: E501

        return self.api_client.call_api(
            '/models/{model_id}/endpoints/{model_endpoint_id}', 'PUT',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='ModelEndpoint',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def models_model_id_endpoints_post(self, body, model_id, **kwargs):  # noqa: E501
        """Create a model endpoint  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.models_model_id_endpoints_post(body, model_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param ModelEndpoint body: Model endpoint object that has to be added (required)
        :param int model_id: (required)
        :return: ModelEndpoint
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.models_model_id_endpoints_post_with_http_info(body, model_id, **kwargs)  # noqa: E501
        else:
            (data) = self.models_model_id_endpoints_post_with_http_info(body, model_id, **kwargs)  # noqa: E501
            return data

    def models_model_id_endpoints_post_with_http_info(self, body, model_id, **kwargs):  # noqa: E501
        """Create a model endpoint  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.models_model_id_endpoints_post_with_http_info(body, model_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param ModelEndpoint body: Model endpoint object that has to be added (required)
        :param int model_id: (required)
        :return: ModelEndpoint
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['body', 'model_id']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method models_model_id_endpoints_post" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'body' is set
        if ('body' not in params or
                params['body'] is None):
            raise ValueError("Missing the required parameter `body` when calling `models_model_id_endpoints_post`")  # noqa: E501
        # verify the required parameter 'model_id' is set
        if ('model_id' not in params or
                params['model_id'] is None):
            raise ValueError("Missing the required parameter `model_id` when calling `models_model_id_endpoints_post`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'model_id' in params:
            path_params['model_id'] = params['model_id']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'body' in params:
            body_params = params['body']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['*/*'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['*/*'])  # noqa: E501

        # Authentication setting
        auth_settings = ['Bearer']  # noqa: E501

        return self.api_client.call_api(
            '/models/{model_id}/endpoints', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='ModelEndpoint',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def projects_project_id_model_endpoints_get(self, project_id, **kwargs):  # noqa: E501
        """List existing model endpoints for all models in particular project  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.projects_project_id_model_endpoints_get(project_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param int project_id: Filter list of model endpoints by specific `project_id` (required)
        :param str region: Filter list of model endpoints by specific environment's `region`
        :return: list[ModelEndpoint]
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.projects_project_id_model_endpoints_get_with_http_info(project_id, **kwargs)  # noqa: E501
        else:
            (data) = self.projects_project_id_model_endpoints_get_with_http_info(project_id, **kwargs)  # noqa: E501
            return data

    def projects_project_id_model_endpoints_get_with_http_info(self, project_id, **kwargs):  # noqa: E501
        """List existing model endpoints for all models in particular project  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.projects_project_id_model_endpoints_get_with_http_info(project_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param int project_id: Filter list of model endpoints by specific `project_id` (required)
        :param str region: Filter list of model endpoints by specific environment's `region`
        :return: list[ModelEndpoint]
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['project_id', 'region']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method projects_project_id_model_endpoints_get" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'project_id' is set
        if ('project_id' not in params or
                params['project_id'] is None):
            raise ValueError("Missing the required parameter `project_id` when calling `projects_project_id_model_endpoints_get`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'project_id' in params:
            path_params['project_id'] = params['project_id']  # noqa: E501

        query_params = []
        if 'region' in params:
            query_params.append(('region', params['region']))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['*/*'])  # noqa: E501

        # Authentication setting
        auth_settings = ['Bearer']  # noqa: E501

        return self.api_client.call_api(
            '/projects/{project_id}/model_endpoints', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='list[ModelEndpoint]',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)



# Copyright 2020 The Merlin Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# coding: utf-8


from __future__ import absolute_import

import sys
import signal
import merlin.fluent
import merlin.resource_request
import merlin.deployment_mode
import merlin.autoscaling
from merlin.version import VERSION as __version__


# Merlin URL
set_url = merlin.fluent.set_url
get_url = merlin.fluent.get_url

# Project
list_project = merlin.fluent.list_project
set_project = merlin.fluent.set_project
active_project = merlin.fluent.active_project

# Environment
list_environment = merlin.fluent.list_environment
get_environment = merlin.fluent.get_environment
get_default_environment = merlin.fluent.get_default_environment

# Model
set_model = merlin.fluent.set_model
active_model = merlin.fluent.active_model

# Model Version
new_model_version = merlin.fluent.new_model_version  # type:ignore
log_param = merlin.fluent.log_param
log_metric = merlin.fluent.log_metric
set_tag = merlin.fluent.set_tag
delete_tag = merlin.fluent.delete_tag
get_param = merlin.fluent.get_param
get_metric = merlin.fluent.get_metric
get_tag = merlin.fluent.get_tag
list_tag = merlin.fluent.list_tag
download_artifact = merlin.fluent.download_artifact

# Log Model Version
log_artifact = merlin.fluent.log_artifact
log_pyfunc_model = merlin.fluent.log_pyfunc_model
log_pytorch_model = merlin.fluent.log_pytorch_model
log_model = merlin.fluent.log_model
log_custom_model = merlin.fluent.log_custom_model

# Model deployment - deploy active model version
deploy = merlin.fluent.deploy
undeploy = merlin.fluent.undeploy

# Model serving
set_traffic = merlin.fluent.set_traffic # deprecated
serve_traffic = merlin.fluent.serve_traffic
stop_serving_traffic = merlin.fluent.stop_serving_traffic

# Model endpoints
list_model_endpoints = merlin.fluent.list_model_endpoints

# Definitions
ResourceRequest = merlin.resource_request.ResourceRequest
DeploymentMode = merlin.deployment_mode.DeploymentMode
AutoscalingPolicy = merlin.autoscaling.AutoscalingPolicy
MetricsType = merlin.autoscaling.MetricsType
# Batch
create_prediction_job = merlin.fluent.create_prediction_job


__all__ = [
    "set_url", "get_url",
    "list_project", "set_project", "active_project",
    "list_environment", "get_environment", "get_default_environment",
    "set_model", "active_model",
    "new_model_version", "log_param", "log_metric", "set_tag", "delete_tag",
    "log_artifact", "log_pyfunc_model", "log_pytorch_model", "log_model",
    "deploy", "undeploy",
    "set_traffic", "serve_traffic",
    "ResourceRequest", "DeploymentMode", "AutoscalingPolicy", "MetricsType",
    "create_prediction_job"
]

def sigterm_handler(_signo, _stack_frame):
    # Raises SystemExit(0):
    sys.exit(0)

signal.signal(signal.SIGTERM, sigterm_handler)